#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, print_function
from gm_rpcd.all import bind
import logging
import traceback
import json
from search.utils.topic import TopicUtils
from libs.es import ESPerform
from libs.cache import redis_client
from search.utils.common import *


def get_home_recommend_topic_ids(user_id,session_id,offset,size,query=None):
    try:
        if query is None:
            redis_key = "physical:home_recommend" + ":user_id:" + str(user_id) + ":session_id:" + session_id
        else:
            redis_key = "physical:home_query" + ":user_id:" + str(user_id) + ":session_id:" + session_id + ":query:" + str(query)

        redis_field_list = [b'last_offset_num', b'unread_topic_id']
        for page_id in range(0,offset,size):
            redis_field_list.append(str(page_id))
        redis_field_val_list = redis_client.hmget(redis_key,redis_field_list)

        last_offset_num = int(redis_field_val_list[0]) if redis_field_val_list[0] else -1
        recommend_topic_ids = []

        topic_id_list = TopicUtils.get_recommend_topic_ids(user_id, offset, size*size,query)
        have_read_group_id_set = set()
        unread_topic_id_list = list()
        have_read_topic_id_set = set()

        ori_unread_topic_id_list = list()

        if redis_field_val_list[1] and offset>0:
            if (user_id>0 and offset==last_offset_num) or user_id==-1:
                ori_unread_topic_id_list = json.loads(redis_field_val_list[1])
                topic_id_list = ori_unread_topic_id_list + topic_id_list

                for have_read_item in redis_field_val_list[2:]:
                    if have_read_item:
                        have_read_topic_id_set=have_read_topic_id_set.union(json.loads(have_read_item))

        # 当前页小组数量
        cur_page_group_num = 0
        # 当前页用户数量
        cur_page_user_num = 0

        for item in topic_id_list:
            if item["group_id"] in have_read_group_id_set:
                unread_topic_id_list.append(item)
            else:
                if item["id"] not in have_read_topic_id_set:
                    if isinstance(item["group_id"],int) and item["group_id"]>0 and cur_page_group_num<(size*0.9):
                        have_read_group_id_set.add(item["group_id"])
                        have_read_topic_id_set.add(item["id"])
                        cur_page_group_num += 1
                        recommend_topic_ids.append(item["id"])
                    elif item["group_id"]<=0 and cur_page_user_num<(size*0.1):
                        cur_page_user_num += 1
                        recommend_topic_ids.append(item["id"])
                    else:
                        unread_topic_id_list.append(item)

                    if len(recommend_topic_ids) >= size:
                        break

        if len(recommend_topic_ids) < size and len(unread_topic_id_list)>0:
            recommend_len = len(recommend_topic_ids)
            offi_unread_topic_id = [item["id"] for item in unread_topic_id_list[:(size-recommend_len)]]
            recommend_topic_ids = recommend_topic_ids + offi_unread_topic_id
            unread_topic_id_list = unread_topic_id_list[(size-recommend_len):]


        if user_id==-1:
            unread_topic_id_list = ori_unread_topic_id_list + unread_topic_id_list
        redis_dict = {
            "unread_topic_id":json.dumps(unread_topic_id_list),
            "last_offset_num":offset+size,
            offset: json.dumps(recommend_topic_ids)
        }
        redis_client.hmset(redis_key,redis_dict)
        # 每个session key保存15分钟
        redis_client.expire(redis_key,15*60*60)

        return recommend_topic_ids
    except:
        logging.error("catch exception,err_msg:%s" % traceback.format_exc())
        return []


@bind("physical/search/home_recommend")
def home_recommend(session_id="",user_id=-1,offset=0,size=10):
    """
    :remark:首页推荐，目前只推荐日记
    :param session_id:
    :param user_id:
    :param offset:
    :param size:
    :return:
    """
    try:
        if not user_id:
            user_id=-1
        if not isinstance(session_id,str):
            session_id = ""

        recommend_topic_ids = get_home_recommend_topic_ids(user_id,session_id,offset,size)

        return {"recommend_topic_ids":recommend_topic_ids}
    except:
        logging.error("catch exception,err_msg:%s" % traceback.format_exc())
        return {"recommend_topic_ids": []}


@bind("physical/search/home_query")
def home_query(session_id="",user_id=-1,query="",offset=0,size=10):
    """
    :remark:首页搜索，目前只推荐日记
    :param session_id:
    :param user_id:
    :param query:
    :param offset:
    :param size:
    :return:
    """
    try:
        if not user_id:
            user_id=-1
        if not isinstance(session_id,str):
            session_id = ""

        recommend_topic_ids = get_home_recommend_topic_ids(user_id,session_id,offset,size,query)
        return {"recommend_topic_ids":recommend_topic_ids}
    except:
        logging.error("catch exception,err_msg:%s" % traceback.format_exc())
        return {"recommend_topic_ids": []}


@bind("physical/search/topic_detail_page_recommend")
def topic_detail_page_recommend(user_id=-1,topic_id=-1,topic_group_id=-1,topic_user_id=-1,offset=0,size=10):
    """
    :remark:帖子详情页推荐策略，缺少第一个卡片策略
    :param user_id:
    :param topic_id:
    :param topic_group_id:
    :param topic_user_id:
    :return:
    """
    try:
        if not isinstance(user_id,int):
            user_id = -1

        # 获取帖子标签列表
        topic_tag_list = TopicUtils.get_topic_tag_id_list(topic_id)

        result_list = TopicUtils.get_topic_detail_recommend_list(user_id,topic_id,topic_tag_list,topic_group_id,topic_user_id,offset,size)
        recommend_topic_ids_list = list()
        if len(result_list)>0:
            recommend_topic_ids_list = [item["_source"]["id"] for item in result_list]

        return {"recommend_topic_ids": recommend_topic_ids_list}
    except:
        logging.error("catch exception,err_msg:%s" % traceback.format_exc())
        return {"recommend_topic_ids": []}


@bind("physical/search/tag_aggregation")
def tag_aggregation(user_id,tag_id, offset, size):
    """
    :remark 按标签聚合召回帖子
    :param tag_id:
    :param offset:
    :param size:
    :return:
    """
    try:
        result_list = TopicUtils.get_tag_aggregation_topic_id_list(user_id,tag_id,offset,size)
        recommend_topic_ids_list = list()
        if len(result_list)>0:
            recommend_topic_ids_list = [item["_source"]["id"] for item in result_list]

        return {"recommend_topic_id": recommend_topic_ids_list}
    except:
        logging.error("catch exception,err_msg:%s" % traceback.format_exc())
        return {"recommend_topic_id": []}


